==> typescript-ioc-0.0.1.tgz: package/src/ioc/IoC.ts <==
/// <reference path="../../node_modules/reflect-metadata/reflect-metadata.d.ts" />
namespace IoC {

    /*********************** Decorators **************************/
    export interface ResourceConfig {
        scope?: Scope;
        bindTypeName?: string;
        provider?: Provider;
    }

    /**
     * Decorator processor for @AutoWired annotation
     */
    export function Resource(config: Object) {
        if (typeof config === "function") {
            return ResourceDecorator(null, <Function>config);
        }

        return function(target: Function) {
            return ResourceDecorator(<ResourceConfig>config, target);
        }
    }

    function ResourceDecorator(config: ResourceConfig, target: Function) {
        let existingInjectedParameters: number[] =
			Reflect.getOwnMetadata("params_inject", target) || [];
        let newConstructor;
        if (existingInjectedParameters.length > 0) {
            const paramTypes: Array<any> =
				Reflect.getMetadata("design:paramtypes", target);
            newConstructor = InjectorHanlder.decorateConstructor(function(...args: any[]) {
                let newArgs: Array<any> = args ? args.concat() : new Array<any>();
                for (let index of existingInjectedParameters) {
                    if (index >= newArgs.length) {
                        newArgs.push(Container.get(paramTypes[index]));
                    }
                }
                target.apply(this, newArgs);
                Container.applyInjections(this, target);
            }, target);
        }
        else {
            newConstructor = InjectorHanlder.decorateConstructor(function(...args: any[]) {
                target.apply(this, args);
                Container.applyInjections(this, target);
            }, target);
        }
        if (config) {
            let Config: Config = Container.bind(newConstructor);
            if ('provider' in config) {
                Config.provider(config.provider);
            }
            else if ('bindTypeName' in config) {
                Config.toTypeNamed(config.bindTypeName);
            }
            else {
                Config.to(newConstructor);
            }
            if (config.scope) {
                Config.scope(config.scope);
            }
        }
        return newConstructor;
    }

    /**
     * Decorator processor for @Inject annotation on properties
     */
    export function Inject(...args: any[]) {
        if (args.length == 2) {
            return InjectPropertyDecorator.apply(this, args);
        }
        else if (args.length == 3 && typeof args[2] === "number") {
			return InjectParamDecorator.apply(this, args);
        }

        throw new Error("Invalid @Inject Decorator declaration.");
    }

    function InjectPropertyDecorator(target: Object, key: string) {
        let t = Reflect.getMetadata("design:type", target, key);
        Container.addPropertyInjector(target.constructor, key, t);
    }

    function InjectParamDecorator(target: Object, propertyKey: string | symbol,
		parameterIndex: number) {
		if (!propertyKey) // only intercept constructor parameters
		{
			let existingInjectedParameters: number[] =
				Reflect.getOwnMetadata("params_inject", target, propertyKey) || [];
			existingInjectedParameters.push(parameterIndex);
			Reflect.defineMetadata("params_inject", existingInjectedParameters,
				target, propertyKey);
		}
    }

    /*********************** Container **************************/
    class Map<V> {
        put(key: any, value: V) {
            this[key] = value;
        }

        get(key: any): V {
            return this[key];
        }
    }

    /**
     * The IoC Container class.
     */
    export class Container {
        private static bindings: Map<ConfigImpl> = new Map<ConfigImpl>();

        static bind(source: Function) {
            checkType(source);
            let config: ConfigImpl = Container.bindings.get(source);
            if (!config) {
                config = new ConfigImpl(source);
                Container.bindings.put(source, config);
            }
            return config;
        }

        static get(source: Function) {
            checkType(source);
            let config: ConfigImpl = Container.bindings.get(source);
            if (!config) {
                config = this.bind(source).to(source);
            }
            return config.getInstance();
        }

        static applyInjections(toInject: Object, targetType?: Function) {
            if (targetType)
            {
				const injections: Array<any> = InjectorHanlder.getInjectorFromType(targetType, false);
				injections.forEach(entry => {
					entry(toInject);
				});
            }
            else{
				const injections: Array<any> = InjectorHanlder.getInjectorFromInstance(toInject);
				injections.forEach(entry => {
					entry(toInject);
				});
            }
        }

        static addPropertyInjector(target: Function, key: string, propertyType: Function) {
            const injections: Array<any> = InjectorHanlder.getInjectorFromType(target, false);
            injections.push(toInject => {
                Container.injectProperty(toInject, key, propertyType);
            });
        }

        static injectProperty(toInject: Object, key: string, source: Function) {
            toInject[key] = Container.get(source);
        }
    }

    function checkType(source: Object) {
        if (!source) {
            throw new TypeError('Invalid type requested to IoC ' +
                'container. Type is not defined.');
        }
    }

    export abstract class Config {
        abstract to(target: Object): Config;
        abstract toTypeNamed(target: string): Config;
        abstract provider(provider: Provider): Config;
        abstract scope(scope: Scope): Config;
    }

    export class ConfigImpl extends Config {
        private source: Object;
        private iocprovider: Provider;
        private iocscope: Scope;

        constructor(source: Object) {
            super();
            this.source = source;
        }

        to(target: any) {
            checkType(target);
            this.iocprovider = function() {
                return new target();
            };
            return this;
        }

        toTypeNamed(target: string) {
            this.iocprovider = function() {
                checkType(window[target]);
                let c = Object.create(window[target].prototype);
                let targetType = c.constructor;
                checkType(targetType);
                return new targetType();
            };

            return this;
        }

        provider(provider: Provider) {
            this.iocprovider = provider;
            return this;
        }

        scope(scope: Scope) {
            this.iocscope = scope;
            return this;
        }

        getInstance() {
            if (!this.iocscope) {
                this.scope(Scope.Local);
            }
            return this.iocscope.resolve(this.iocprovider, this.source);
        }
    }

    export interface Provider {
        (): Object;
    }

    export abstract class Scope {
        static Local: Scope;
        static Singleton: Scope;

        abstract resolve(provider: Provider, source: Object): any;
    }

    export class LocalScope extends Scope {
        resolve(provider: Provider, source: Object) {
            return provider();
        }
    }

    Scope.Local = new LocalScope();

    export class SingletonScope extends Scope {
        private static instances: Map<any> = new Map<any>();

        resolve(provider: Provider, source: Object) {
            let instance: any = SingletonScope.instances.get(source);
            if (!instance) {
                instance = provider();
                SingletonScope.instances.put(source, instance);
            }
            return instance;
        }
    }

    Scope.Singleton = new SingletonScope();


	/**
	 * Utility class to handle injection behavior on class decorations.
	 */
    class InjectorHanlder {
		static typeInjections: Map<Array<any>> = new Map<Array<any>>();

		static decorateConstructor(derived: Function, base: Function) {
			for (var p in base) {
				if (base.hasOwnProperty(p) && !derived.hasOwnProperty(p)) {
					derived[p] = base[p];
				}
			}
			derived['__parent'] = base;
			function __() { this.constructor = derived; }
			derived.prototype = base === null ? Object.create(base) :
				(__.prototype = base.prototype, new __());
			return derived;
		}

		static getConstructorFromType(target: Function) {
			let typeConstructor: Function = target;
			if (typeConstructor['name'])
			{
				return typeConstructor;
			}
			while (typeConstructor = typeConstructor['__parent'])
			{
				if (typeConstructor['name']) {
					return typeConstructor;
				}
			}
			throw TypeError('Can not identify the base Type for requested target');
		}

		static getConstructorFromInstance(target: Object) {
			return InjectorHanlder.getConstructorFromType(target.constructor);
		}
		static getInjectorFromType(target: Function, recursive: boolean) {
			const baseConstructor: Object = InjectorHanlder.getConstructorFromType(target);
			if (!InjectorHanlder.typeInjections[<any>baseConstructor]) {
				InjectorHanlder.typeInjections[<any>baseConstructor] = new Array<any>();
			}
			let result: Array<any> = InjectorHanlder.typeInjections[<any>baseConstructor];
			if (recursive)
			{
				let parent: Function = baseConstructor['__parent'];
				if (parent)
				{
					result = InjectorHanlder.getInjectorFromType(parent, true).concat(result);
				}
			}
			return result;
		}

		static getInjectorFromInstance(target: Object) {
			const baseConstructor: Function = InjectorHanlder.getConstructorFromInstance(target);
			return InjectorHanlder.getInjectorFromType(baseConstructor, true)
		}
	}
}

==> typescript-ioc-0.0.3.tgz: package/src/ioc/IoC.ts <==
/// <reference path="../../node_modules/reflect-metadata/reflect-metadata.d.ts" />
namespace IoC {

    /*********************** Decorators **************************/
    export interface ResourceConfig {
        scope?: Scope;
        bindTypeName?: string;
        provider?: Provider;
    }

    /**
     * Decorator processor for @AutoWired annotation
     */
    export function Resource(config: Object) {
        if (typeof config === "function") {
            return ResourceDecorator(null, <Function>config);
        }

        return function(target: Function) {
            return ResourceDecorator(<ResourceConfig>config, target);
        }
    }

    function ResourceDecorator(config: ResourceConfig, target: Function) {
        let existingInjectedParameters: number[] =
			Reflect.getOwnMetadata("params_inject", target) || [];
        let newConstructor;
        if (existingInjectedParameters.length > 0) {
            const paramTypes: Array<any> =
				Reflect.getMetadata("design:paramtypes", target);
            newConstructor = InjectorHanlder.decorateConstructor(function(...args: any[]) {
                let newArgs: Array<any> = args ? args.concat() : new Array<any>();
                for (let index of existingInjectedParameters) {
                    if (index >= newArgs.length) {
                        newArgs.push(Container.get(paramTypes[index]));
                    }
                }
                target.apply(this, newArgs);
                Container.applyInjections(this, target);
            }, target);
        }
        else {
            newConstructor = InjectorHanlder.decorateConstructor(function(...args: any[]) {
                target.apply(this, args);
                Container.applyInjections(this, target);
            }, target);
        }
        if (config) {
            let Config: Config = Container.bind(newConstructor);
            if ('provider' in config) {
                Config.provider(config.provider);
            }
            else if ('bindTypeName' in config) {
                Config.toTypeNamed(config.bindTypeName);
            }
            else {
                Config.to(newConstructor);
            }
            if (config.scope) {
                Config.scope(config.scope);
            }
        }
        return newConstructor;
    }

    /**
     * Decorator processor for @Inject annotation on properties
     */
    export function Inject(...args: any[]) {
        if (args.length == 2) {
            return InjectPropertyDecorator.apply(this, args);
        }
        else if (args.length == 3 && typeof args[2] === "number") {
			return InjectParamDecorator.apply(this, args);
        }

        throw new Error("Invalid @Inject Decorator declaration.");
    }

    function InjectPropertyDecorator(target: Object, key: string) {
        let t = Reflect.getMetadata("design:type", target, key);
        Container.addPropertyInjector(target.constructor, key, t);
    }

    function InjectParamDecorator(target: Object, propertyKey: string | symbol,
		parameterIndex: number) {
		if (!propertyKey) // only intercept constructor parameters
		{
			let existingInjectedParameters: number[] =
				Reflect.getOwnMetadata("params_inject", target, propertyKey) || [];
			existingInjectedParameters.push(parameterIndex);
			Reflect.defineMetadata("params_inject", existingInjectedParameters,
				target, propertyKey);
		}
    }

    /*********************** Container **************************/
    class Map<V> {
        put(key: any, value: V) {
            this[key] = value;
        }

        get(key: any): V {
            return this[key];
        }
    }

    /**
     * The IoC Container class.
     */
    export class Container {
        private static bindings: Map<ConfigImpl> = new Map<ConfigImpl>();

        static bind(source: Function) {
            checkType(source);
            let config: ConfigImpl = Container.bindings.get(source);
            if (!config) {
                config = new ConfigImpl(source);
                Container.bindings.put(source, config);
            }
            return config;
        }

        static get(source: Function) {
            checkType(source);
            let config: ConfigImpl = Container.bindings.get(source);
            if (!config) {
                config = this.bind(source).to(source);
            }
            return config.getInstance();
        }

        static applyInjections(toInject: Object, targetType?: Function) {
            if (targetType)
            {
				const injections: Array<any> = InjectorHanlder.getInjectorFromType(targetType, false);
				injections.forEach(entry => {
					entry(toInject);
				});
            }
            else{
				const injections: Array<any> = InjectorHanlder.getInjectorFromInstance(toInject);
				injections.forEach(entry => {
					entry(toInject);
				});
            }
        }

        static addPropertyInjector(target: Function, key: string, propertyType: Function) {
            const injections: Array<any> = InjectorHanlder.getInjectorFromType(target, false);
            injections.push(toInject => {
                Container.injectProperty(toInject, key, propertyType);
            });
        }

        static injectProperty(toInject: Object, key: string, source: Function) {
            toInject[key] = Container.get(source);
        }
    }

    function checkType(source: Object) {
        if (!source) {
            throw new TypeError('Invalid type requested to IoC ' +
                'container. Type is not defined.');
        }
    }

    export abstract class Config {
        abstract to(target: Object): Config;
        abstract toTypeNamed(target: string): Config;
        abstract provider(provider: Provider): Config;
        abstract scope(scope: Scope): Config;
    }

    export class ConfigImpl extends Config {
        private source: Function;
        private iocprovider: Provider;
        private iocscope: Scope;

        constructor(source: Function) {
            super();
            this.source = source;
        }

        to(target: any) {
            checkType(target);
            this.iocprovider = function() {
                return new target();
            };
            return this;
        }

        toTypeNamed(target: string) {
            this.iocprovider = function() {
                checkType(window[target]);
                let c = Object.create(window[target].prototype);
                let targetType = c.constructor;
                checkType(targetType);
                return new targetType();
            };

            return this;
        }

        provider(provider: Provider) {
            this.iocprovider = provider;
            return this;
        }

        scope(scope: Scope) {
            this.iocscope = scope;
            return this;
        }

        getInstance() {
            if (!this.iocscope) {
                this.scope(Scope.Local);
            }
            return this.iocscope.resolve(this.iocprovider, this.source);
        }
    }

    export interface Provider {
        (): Object;
    }

    export abstract class Scope {
        static Local: Scope;
        static Singleton: Scope;

        abstract resolve(provider: Provider, source: Function): any;
    }

    export class LocalScope extends Scope {
        resolve(provider: Provider, source: Function) {
            return provider();
        }
    }

    Scope.Local = new LocalScope();

    export class SingletonScope extends Scope {
        private static instances: Map<any> = new Map<any>();

        resolve(provider: Provider, source: Function) {
            let instance: any = SingletonScope.instances.get(source);
            if (!instance) {
                instance = provider();
                SingletonScope.instances.put(source, instance);
            }
            return instance;
        }
    }

    Scope.Singleton = new SingletonScope();


	/**
	 * Utility class to handle injection behavior on class decorations.
	 */
    class InjectorHanlder {
		static typeInjections: Map<Array<any>> = new Map<Array<any>>();

		static decorateConstructor(derived: Function, base: Function) {
			for (var p in base) {
				if (base.hasOwnProperty(p) && !derived.hasOwnProperty(p)) {
					derived[p] = base[p];
				}
			}
			derived['__parent'] = base;
			function __() { this.constructor = derived; }
			derived.prototype = base === null ? Object.create(base) :
				(__.prototype = base.prototype, new __());
			return derived;
		}

		static getConstructorFromType(target: Function) {
			let typeConstructor: Function = target;
			if (typeConstructor['name'])
			{
				return typeConstructor;
			}
			while (typeConstructor = typeConstructor['__parent'])
			{
				if (typeConstructor['name']) {
					return typeConstructor;
				}
			}
			throw TypeError('Can not identify the base Type for requested target');
		}

		static getConstructorFromInstance(target: Object) {
			return InjectorHanlder.getConstructorFromType(target.constructor);
		}
		static getInjectorFromType(target: Function, recursive: boolean) {
			const baseConstructor: Object = InjectorHanlder.getConstructorFromType(target);
			if (!InjectorHanlder.typeInjections[<any>baseConstructor]) {
				InjectorHanlder.typeInjections[<any>baseConstructor] = new Array<any>();
			}
			let result: Array<any> = InjectorHanlder.typeInjections[<any>baseConstructor];
			if (recursive)
			{
				let parent: Function = baseConstructor['__parent'];
				if (parent)
				{
					result = InjectorHanlder.getInjectorFromType(parent, true).concat(result);
				}
			}
			return result;
		}

		static getInjectorFromInstance(target: Object) {
			const baseConstructor: Function = InjectorHanlder.getConstructorFromInstance(target);
			return InjectorHanlder.getInjectorFromType(baseConstructor, true)
		}
	}
}
